@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/*$Id: gdevdbit.c 10527 2009-12-18 22:24:19Z giles $ */
+/*$Id: gdevdbit.c 11581 2010-07-31 15:45:39Z chrisl $ */
 /* Default device bitmap copying implementation */
 #include "gx.h"
 #include "gpcheck.h"
@@ -289,7 +289,7 @@ gx_default_copy_alpha(gx_device * dev, const byte * data, int data_x,
 		}
 		LINE_ACCUM(composite, bpp);
 	    }
-	    LINE_ACCUM_COPY(dev, lout, bpp, x, rx, raster, ry);
+	    LINE_ACCUM_COPY(dev, lout, bpp, x, rx, out_size, ry);
 	}
       out:gs_free_object(mem, lout, "copy_alpha(lout)");
 	gs_free_object(mem, lin, "copy_alpha(lin)");
