@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfi.c 10087 2009-09-25 09:12:10Z ken $ */
+/* $Id: gdevpdfi.c 11908 2010-11-23 08:46:10Z ken $ */
 /* Image handling for PDF-writing driver */
 #include "memory_.h"
 #include "math_.h"
@@ -258,7 +258,7 @@ make_device_color_space(gx_device_pdf *pdev,
 	       Don't emit rangecheck becuause it would fall back
 	       to a default implementation (rasterisation). 
 	     */
-	    eprintf("Unsupported ProcessColorModel");
+	    emprintf(mem, "Unsupported ProcessColorModel");
 	    return_error(gs_error_undefined);
     }
     if (cs == NULL)
@@ -363,7 +363,7 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
 	is_mask = pim1->ImageMask;
 	if (is_mask) {
 	    /* If parameters are invalid, use the default implementation. */
-	    if (pdcolor->type != &gx_dc_pattern)
+	    if (!(gx_dc_is_pattern1_color(pdcolor)))
 		if (pim1->BitsPerComponent != 1 ||
 		    !((pim1->Decode[0] == 0.0 && pim1->Decode[1] == 1.0) ||
 		      (pim1->Decode[0] == 1.0 && pim1->Decode[1] == 0.0))
@@ -664,7 +664,10 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
 	        convert_to_process_colors = true;
         }
     }
-    if (image[0].pixel.ColorSpace != NULL) { /* Not an imagemask. */
+    if (image[0].pixel.ColorSpace != NULL && !(context == PDF_IMAGE_TYPE3_MASK)) { 
+	/* Not an imagemask. Also not a SMask, we carefully made a Gray space
+	 * for SMasks above, do not destroy it now!
+	 */
 	if ((pdev->params.ColorConversionStrategy == ccs_Gray &&
 	     !check_image_color_space(&image[0].pixel, gs_color_space_index_DeviceGray)) ||
 	    (pdev->params.ColorConversionStrategy == ccs_sRGB &&
@@ -684,7 +687,8 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
 		 strcmp(pdev->color_info.cm_name, "DeviceRGB")) ||
 		(pdev->params.ColorConversionStrategy == ccs_Gray && 
 		 strcmp(pdev->color_info.cm_name, "DeviceGray"))) {
-		eprintf("ColorConversionStrategy isn't compatible to ProcessColorModel.");
+		emprintf(pdev->memory, "ColorConversionStrategy isn't "
+                         "compatible to ProcessColorModel.");
 		return_error(gs_error_rangecheck);
 	    }
 	    convert_to_process_colors = true;
@@ -698,7 +702,7 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
 	    case gs_color_space_index_DeviceRGB:  sname = names->DeviceRGB;  break;
 	    case gs_color_space_index_DeviceCMYK: sname = names->DeviceCMYK; break;
 	    default:
-		eprintf("Unsupported ProcessColorModel.");
+		emprintf(pdev->memory, "Unsupported ProcessColorModel.");
 		return_error(gs_error_undefined);
 	}
 	cos_c_string_value(&cs_value, sname);
@@ -709,7 +713,21 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
 	image[0].pixel.ColorSpace = pcs_device;
     }
     pdev->ParamCompatibilityLevel = pdev->CompatibilityLevel;
-    if ((code = pdf_begin_write_image(pdev, &pie->writer, gs_no_id, width,
+    
+    /* This is rather hacky. the gs_image_pixel_t union has copies of the image
+     * information, including the ColorSpace, but it does not increment the
+     * reference counts of the counted objects (teh ColorSpace) when it makes
+     * copies. However, if psdf_setup_image_filters() should change the colour
+     * space into DeviceCMYK because it converts the image, it will decrement
+     * the reference count of the original space. There is at least one place
+     * where it is called with the original space, so we can't simply remove
+     * the decrement. Handling this properly would entail incrementing the
+     * reference count when we make the copy, and correctly decrementing it
+     * in all the error conditions. Its easier to isolate the changes to
+     * this piece of code.
+     */
+    rc_increment_cs(image[0].pixel.ColorSpace);
+    if ((pdf_begin_write_image(pdev, &pie->writer, gs_no_id, width,
 		    height, pnamed, in_line)) < 0 ||
 	/*
 	 * Some regrettable PostScript code (such as LanguageLevel 1 output
@@ -720,15 +738,21 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
 	 * filters if the transfer function(s) is/are other than the
 	 * identity.
 	 */
-	(code = (pie->writer.alt_writer_count == 1 ?
+	((pie->writer.alt_writer_count == 1 ?
 		 psdf_setup_lossless_filters((gx_device_psdf *) pdev,
 					     &pie->writer.binary[0],
 					     &image[0].pixel, in_line) :
 		 psdf_setup_image_filters((gx_device_psdf *) pdev,
 					  &pie->writer.binary[0], &image[0].pixel,
 					  pmat, pis, true, in_line))) < 0
-	)
+	) {
+	if (image[0].pixel.ColorSpace == pim->ColorSpace)
+	    rc_decrement_only_cs(pim->ColorSpace, "psdf_setup_image_filters");
 	goto fail;
+    }
+    if (image[0].pixel.ColorSpace == pim->ColorSpace)
+        rc_decrement_only_cs(pim->ColorSpace, "psdf_setup_image_filters");
+
     if (convert_to_process_colors) {
 	image[0].pixel.ColorSpace = pcs_orig;
 	code = psdf_setup_image_colors_filter(&pie->writer.binary[0], 
@@ -737,6 +761,9 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
   	    goto fail;
 	image[0].pixel.ColorSpace = pcs_device;
     }
+
+    /* See the comment above about reference counting of the colour space */
+    rc_increment_cs(image[1].pixel.ColorSpace);
     if (pie->writer.alt_writer_count > 1) {
         code = pdf_make_alt_stream(pdev, &pie->writer.binary[1]);
         if (code)
@@ -751,17 +778,26 @@ pdf_begin_typed_image_impl(gx_device_pdf *pdev, const gs_imager_state * pis,
 	    pie->writer.alt_writer_count = 1;
 	    memset(pie->writer.binary + 1, 0, sizeof(pie->writer.binary[1]));
 	    memset(pie->writer.binary + 2, 0, sizeof(pie->writer.binary[1]));
-	} else if (code)
+	} else if (code) {
+	    if (image[1].pixel.ColorSpace == pim->ColorSpace)
+		rc_decrement_only_cs(pim->ColorSpace, "psdf_setup_image_filters");
 	    goto fail;
+	}
 	else if (convert_to_process_colors) {
 	    image[1].pixel.ColorSpace = pcs_orig;
 	    code = psdf_setup_image_colors_filter(&pie->writer.binary[1], 
 		    (gx_device_psdf *)pdev, &image[1].pixel, pis);
-	    if (code < 0)
+	    if (code < 0) {
+		if (image[1].pixel.ColorSpace == pim->ColorSpace)
+		    rc_decrement_only_cs(pim->ColorSpace, "psdf_setup_image_filters");
   		goto fail;
+	    }
 	    image[1].pixel.ColorSpace = pcs_device;
 	}
     }
+    if (image[1].pixel.ColorSpace == pim->ColorSpace)
+        rc_decrement_only_cs(pim->ColorSpace, "psdf_setup_image_filters");
+
     for (i = 0; i < pie->writer.alt_writer_count; i++) {
 	code = pdf_begin_image_data_decoded(pdev, num_components, pranges, i,
 			     &image[i].pixel, &cs_value, pie);
@@ -1241,6 +1277,8 @@ pdf_image3_make_mcde(gx_device *dev, const gs_imager_state *pis,
 	code = gx_default_begin_typed_image
 	    ((gx_device *)&cvd->mdev, pis, pmat, pic, prect, pdcolor, NULL, mem,
 	    pinfo);
+	if (code < 0)
+	    return code;
     } else {
 	code = pdf_make_mxd(pmcdev, midev, mem);
 	if (code < 0)
@@ -1248,6 +1286,8 @@ pdf_image3_make_mcde(gx_device *dev, const gs_imager_state *pis,
 	code = pdf_begin_typed_image
 	    ((gx_device_pdf *)dev, pis, pmat, pic, prect, pdcolor, pcpath, mem,
 	    pinfo, PDF_IMAGE_TYPE3_DATA);
+	if (code < 0)
+	    return code;
     }
     /* Due to equal image merging, we delay the adding of the "Mask" entry into 
        a type 3 image dictionary until the mask is completed. 
@@ -1285,8 +1325,6 @@ pdf_image3x_make_mcde(gx_device *dev, const gs_imager_state *pis,
 {
     int code;
     pdf_image_enum *pmie;
-    pdf_image_enum *pmce;
-    cos_stream_t *pmcs;
     int i;
     const gs_image3x_mask_t *pixm;
 
@@ -1313,8 +1351,6 @@ pdf_image3x_make_mcde(gx_device *dev, const gs_imager_state *pis,
 	return_error(gs_error_rangecheck);
     }
     pmie = (pdf_image_enum *)pminfo[i];
-    pmce = (pdf_image_enum *)(*pinfo);
-    pmcs = (cos_stream_t *)pmce->writer.pres->object;
     /*
      * Add the SMask entry to the image dictionary, and, if needed,
      * the Matte entry to the mask dictionary.
