@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfm.c 10141 2009-10-06 07:33:33Z ken $ */
+/* $Id: gdevpdfm.c 11946 2010-12-10 13:26:11Z ken $ */
 /* pdfmark processing for PDF-writing driver */
 #include "math_.h"
 #include "memory_.h"
@@ -125,7 +125,7 @@ pdfmark_page_number(gx_device_pdf * pdev, const gs_param_string * pnstr)
 static int
 pdfmark_make_dest(char dstr[MAX_DEST_STRING], gx_device_pdf * pdev,
 		  const char *Page_key, const char *View_key,
-		  const gs_param_string * pairs, uint count)
+		  const gs_param_string * pairs, uint count, uint RequirePage)
 {
     gs_param_string page_string, view_string;
     int present =
@@ -135,7 +135,7 @@ pdfmark_make_dest(char dstr[MAX_DEST_STRING], gx_device_pdf * pdev,
     gs_param_string action;
     int len;
 
-    if (present)
+    if (present || RequirePage)
 	page = pdfmark_page_number(pdev, &page_string);
     if (view_string.size == 0)
 	param_string_from_string(view_string, "[/XYZ null null null]");
@@ -357,7 +357,7 @@ pdfmark_bind_named_object(gx_device_pdf *pdev, const gs_const_string *objname,
 		} else if (!v->contents.object->written) {
 		    /* We can't know whether the old object was referred or not.
 		       Write it out for a consistent result in any case. */
-		    code = cos_write_object(v->contents.object, pdev);
+		    code = cos_write_object(v->contents.object, pdev, resourceOther);
 
 		    if (code < 0)
 			return code;
@@ -485,7 +485,7 @@ pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 	    /* Make a destination even if this is for an outline. */
 	    if (Dest.data == 0) {
 		code = pdfmark_make_dest(dest, params->pdev, "/Page", "/View",
-					 pairs, count);
+					 pairs, count, 0);
 		if (code < 0)
 		    return code;
 		param_string_from_string(Dest, dest);
@@ -723,7 +723,8 @@ pdfmark_annot(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	    if (pdf_key_eq(pair, "/F")) {
 		code = sscanf((const char *)pair[1].data, "%ld", &Flags);
 		if (code != 1)
-		    eprintf("Annotation has an invalid /Flags attribute\n");
+		    emprintf(pdev->memory,
+                             "Annotation has an invalid /Flags attribute\n");
 		break;
 	    }
 	}
@@ -734,7 +735,8 @@ pdfmark_annot(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		 * output file will not be PDF/A compliant
 		 */
 		case 0:
-		    eprintf("Annotation set to non-printing,\n not permitted in PDF/A, reverting to normal PDF output\n");
+		    emprintf(pdev->memory,
+                             "Annotation set to non-printing,\n not permitted in PDF/A, reverting to normal PDF output\n");
 		    pdev->AbortPDFAX = true;
 		    pdev->PDFA = false;
 		    break;
@@ -742,11 +744,18 @@ pdfmark_annot(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		     * include it, but warn the user that it has been dropped.
 		     */
 		case 1:
-		    eprintf("Annotation set to non-printing,\n not permitted in PDF/A, annotation will not be present in output file\n");
+		    emprintf(pdev->memory,
+                             "Annotation set to non-printing,\n not permitted in PDF/A, annotation will not be present in output file\n");
 		    return 0;
 		    break;
+		case 2:
+		    emprintf(pdev->memory,
+                             "Annotation set to non-printing,\n not permitted in PDF/A, aborting conversion\n");
+		    return gs_error_invalidfont;
+		    break;
 		default:
-		    eprintf("Annotation set to non-printing,\n not permitted in PDF/A, unrecognised PDFACompatibilityLevel,\nreverting to normal PDF output\n");
+		    emprintf(pdev->memory,
+                             "Annotation set to non-printing,\n not permitted in PDF/A, unrecognised PDFACompatibilityLevel,\nreverting to normal PDF output\n");
 		    pdev->AbortPDFAX = true;
 		    pdev->PDFA = false;
 		    break;
@@ -778,7 +787,7 @@ pdfmark_annot(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
     }
     if (!objname) {
 	/* Write the annotation now. */
-	COS_WRITE_OBJECT(pcd, pdev);
+	COS_WRITE_OBJECT(pcd, pdev, resourceAnnotation);
 	COS_RELEASE(pcd, "pdfmark_annot");
     }
     return cos_array_add(annots,
@@ -809,11 +818,13 @@ pdfmark_write_outline(gx_device_pdf * pdev, pdf_outline_node_t * pnode,
     stream *s;
     int code = 0;
 
-    pdf_open_separate(pdev, pnode->id);
+    pdf_open_separate(pdev, pnode->id, resourceOutline);
     if (pnode->action != NULL)
 	pnode->action->id = pnode->id;
     else {
-	eprintf1("pdfmark error: Outline node %ld has no action or destination.\n", pnode->id);
+	emprintf1(pdev->memory,
+                  "pdfmark error: Outline node %ld has no action or destination.\n",
+                  pnode->id);
 	code = gs_note_error(gs_error_undefined);
     }
     s = pdev->strm;
@@ -831,7 +842,7 @@ pdfmark_write_outline(gx_device_pdf * pdev, pdf_outline_node_t * pnode,
 	pprintld2(s, "/First %ld 0 R /Last %ld 0 R\n",
 		  pnode->first_id, pnode->last_id);
     stream_puts(s, ">>\n");
-    pdf_end_separate(pdev);
+    pdf_end_separate(pdev, resourceOutline);
     if (pnode->action != NULL)
 	COS_FREE(pnode->action, "pdfmark_write_outline");
     pnode->action = 0;
@@ -959,7 +970,7 @@ pdfmark_write_bead(gx_device_pdf * pdev, const pdf_bead_t * pbead)
     stream *s;
     char rstr[MAX_RECT_STRING];
 
-    pdf_open_separate(pdev, pbead->id);
+    pdf_open_separate(pdev, pbead->id, resourceArticle);
     s = pdev->strm;
     pprintld3(s, "<</T %ld 0 R/V %ld 0 R/N %ld 0 R",
 	      pbead->article_id, pbead->prev_id, pbead->next_id);
@@ -967,7 +978,7 @@ pdfmark_write_bead(gx_device_pdf * pdev, const pdf_bead_t * pbead)
 	pprintld1(s, "/P %ld 0 R", pbead->page_id);
     pdfmark_make_rect(rstr, &pbead->rect);
     pprints1(s, "/R%s>>\n", rstr);
-    return pdf_end_separate(pdev);
+    return pdf_end_separate(pdev, resourceArticle);
 }
 
 /* Finish writing an article, and release its data. */
@@ -988,12 +999,12 @@ pdfmark_write_article(gx_device_pdf * pdev, const pdf_article_t * part)
 	pdfmark_write_bead(pdev, &art.last);
     }
     pdfmark_write_bead(pdev, &art.first);
-    pdf_open_separate(pdev, art.contents->id);
+    pdf_open_separate(pdev, art.contents->id, resourceArticle);
     s = pdev->strm;
     pprintld1(s, "<</F %ld 0 R/I<<", art.first.id);
     cos_dict_elements_write(art.contents, pdev);
     stream_puts(s, ">> >>\n");
-    return pdf_end_separate(pdev);
+    return pdf_end_separate(pdev, resourceArticle);
 }
 
 /* ARTICLE pdfmark */
@@ -1100,7 +1111,7 @@ pdfmark_DEST(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 
     if (!pdfmark_find_key("/Dest", pairs, count, &key) ||
 	(present =
-	 pdfmark_make_dest(dest, pdev, "/Page", "/View", pairs, count)) < 0
+	 pdfmark_make_dest(dest, pdev, "/Page", "/View", pairs, count, 1)) < 0
 	)
 	return_error(gs_error_rangecheck);
     cos_string_value(&value, (byte *)dest, strlen(dest));
@@ -1138,7 +1149,7 @@ pdfmark_DEST(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 
 /* Check that pass-through PostScript code is a string. */
 static bool
-ps_source_ok(const gs_param_string * psource)
+ps_source_ok(const gs_memory_t *mem, const gs_param_string * psource)
 {
     if (psource->size >= 2 && psource->data[0] == '(' &&
 	psource->data[psource->size - 1] == ')'
@@ -1148,8 +1159,8 @@ ps_source_ok(const gs_param_string * psource)
 	int i;
 	lprintf("bad PS passthrough: ");
 	for (i=0; i<psource->size; i++)
-	    errprintf("%c", psource->data[i]);
-	errprintf("\n");
+	    errprintf(mem, "%c", psource->data[i]);
+	errprintf(mem, "\n");
 	return false;
     }
 }
@@ -1208,9 +1219,9 @@ pdfmark_PS(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
     gs_param_string level1;
 
     if (!pdfmark_find_key("/DataSource", pairs, count, &source) ||
-	!ps_source_ok(&source) ||
+	!ps_source_ok(pdev->memory, &source) ||
 	(pdfmark_find_key("/Level1", pairs, count, &level1) &&
-	 !ps_source_ok(&level1))
+	 !ps_source_ok(pdev->memory, &level1))
 	)
 	return_error(gs_error_rangecheck);
     if (level1.data == 0 && source.size <= MAX_PS_INLINE && objname == 0) {
@@ -1252,7 +1263,7 @@ pdfmark_PS(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	    code = pdf_exit_substream(pdev);
 	    if (code < 0)
 		return code;
-	    code = cos_write_object(pres->object, pdev);
+	    code = cos_write_object(pres->object, pdev, resourceOther);
 	    if (code < 0)
 		return code;
 	    level1_id = pres->object->id;
@@ -1501,7 +1512,7 @@ pdfmark_DOCINFO(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 		    return_error(gs_error_VMerror);
 		memcpy(str, vdata, j);
 		memcpy(str + j, buf + 1, len);
-		alt_pair[1].data = vdata = str;
+		alt_pair[1].data = str;
 		alt_pair[1].size = vsize = j + len;
 		pair = alt_pair;
 	    }
@@ -1523,7 +1534,7 @@ pdfmark_DOCVIEW(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 
     if (count & 1)
 	return_error(gs_error_rangecheck);
-    if (pdfmark_make_dest(dest, pdev, "/Page", "/View", pairs, count)) {
+    if (pdfmark_make_dest(dest, pdev, "/Page", "/View", pairs, count, 0)) {
 	int i;
 
 	code = cos_dict_put_c_key_string(pdev->Catalog, "/OpenAction",
