@@ -11,10 +11,9 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdfo.c 9114 2008-09-22 10:56:52Z ken $ */
+/* $Id: gdevpdfo.c 11946 2010-12-10 13:26:11Z ken $ */
 /* Cos object support */
 #include "memory_.h"
-#include "string_.h"
 #include "gx.h"
 #include "gserrors.h"
 #include "gsparam.h"
@@ -220,15 +219,15 @@ cos_write(const cos_object_t *pco, gx_device_pdf *pdev, gs_id object_id)
 
 /* Write a cos object as a PDF object. */
 int
-cos_write_object(cos_object_t *pco, gx_device_pdf *pdev)
+cos_write_object(cos_object_t *pco, gx_device_pdf *pdev, pdf_resource_type_t type)
 {
     int code;
 
     if (pco->id == 0 || pco->written)
 	return_error(gs_error_Fatal);
-    pdf_open_separate(pdev, pco->id);
+    pdf_open_separate(pdev, pco->id, type);
     code = cos_write(pco, pdev, pco->id);
-    pdf_end_separate(pdev);
+    pdf_end_separate(pdev, type);
     pco->written = true;
     return code;
 }
@@ -509,10 +508,42 @@ cos_array_write(const cos_object_t *pco, gx_device_pdf *pdev, gs_id object_id)
     const cos_array_t *const pca = (const cos_array_t *)pco;
     cos_array_element_t *first = cos_array_reorder(pca, NULL);
     cos_array_element_t *pcae;
-    uint last_index = 0;
+    uint last_index = 0, Element_Count = 0;
 
     stream_puts(s, "[");
     for (pcae = first; pcae; ++last_index, pcae = pcae->next) {
+	Element_Count++;
+
+	if(pdev->PDFA && Element_Count > 8191) {
+	    switch (pdev->PDFACompatibilityPolicy) {
+	    case 0:
+		emprintf(pdev->memory,
+		    "Too many entries in array,\n max 8191 in PDF/A, reverting to normal PDF output\n");
+		pdev->AbortPDFAX = true;
+		pdev->PDFA = false;
+		break;
+	    case 1:
+		emprintf(pdev->memory,
+                     "Too many entries in array,\n max 8191 in PDF/A. Cannot simply elide dictionary, reverting to normal output\n");
+		pdev->AbortPDFAX = true;
+		pdev->PDFA = false;
+		break;
+	    case 2:
+		emprintf(pdev->memory,
+                     "Too many entries in array,\n max 8191 in PDF/A. aborting conversion\n");
+		/* Careful here, only certain errors will bubble up 
+		 * through the text processing.
+		 */
+		return gs_error_limitcheck;
+		break;
+	    default:
+		emprintf(pdev->memory,
+                     "Too many entries in array,\n max 8191 in PDF/A. Unrecognised PDFACompatibilityLevel,\nreverting to normal PDF output\n");
+		pdev->AbortPDFAX = true;
+		pdev->PDFA = false;
+		break;
+	    }
+	}
 	if (pcae != first)
 	    stream_putc(s, '\n');
 	for (; pcae->index > last_index; ++last_index)
@@ -752,6 +783,8 @@ static int
 cos_elements_write(stream *s, const cos_dict_element_t *pcde,
 		   gx_device_pdf *pdev, bool do_space, gs_id object_id)
 {
+    int Element_Count = 0;
+
     if (pcde) {
 	/* Temporarily replace the output stream in pdev. */
 	stream *save = pdev->strm;
@@ -763,6 +796,39 @@ cos_elements_write(stream *s, const cos_dict_element_t *pcde,
 				    pcde->key.data, pcde->key.size) 
 				? object_id : (gs_id)-1);
 
+	    Element_Count++;
+
+	    if(pdev->PDFA && Element_Count > 4095) {
+		switch (pdev->PDFACompatibilityPolicy) {
+		    case 0:
+			emprintf(pdev->memory,
+                             "Too many entries in dictionary,\n max 4095 in PDF/A, reverting to normal PDF output\n");
+			pdev->AbortPDFAX = true;
+			pdev->PDFA = false;
+			break;
+		    case 1:
+			emprintf(pdev->memory,
+                             "Too many entries in dictionary,\n max 4095 in PDF/A. Cannot simply elide dictionary, reverting to normal output\n");
+			pdev->AbortPDFAX = true;
+			pdev->PDFA = false;
+			break;
+		    case 2:
+			emprintf(pdev->memory,
+                             "Too many entries in dictionary,\n max 4095 in PDF/A. aborting conversion\n");
+			/* Careful here, only certain errors will bubble up 
+			 * through the text processing.
+			 */
+			return gs_error_limitcheck;
+			break;
+		    default:
+			emprintf(pdev->memory,
+                             "Too many entries in dictionary,\n max 4095 in PDF/A. Unrecognised PDFACompatibilityLevel,\nreverting to normal PDF output\n");
+			pdev->AbortPDFAX = true;
+			pdev->PDFA = false;
+			break;
+		}
+	    }
+
 	    pdf_write_value(pdev, pcde->key.data, pcde->key.size, object_id1);
 	    cos_value_write_spaced(&pcde->value, pdev, true, object_id1);
 	    pcde = pcde->next;
@@ -805,7 +871,7 @@ cos_dict_objects_write(const cos_dict_t *pcd, gx_device_pdf *pdev)
 	if (COS_VALUE_IS_OBJECT(&pcde->value) &&
 	    pcde->value.contents.object->id  &&
 	    !pcde->value.contents.object->written /* ForOPDFRead only. */)
-	    cos_write_object(pcde->value.contents.object, pdev);
+	    cos_write_object(pcde->value.contents.object, pdev, resourceOther);
     return 0;
 }
 int
@@ -1213,6 +1279,9 @@ cos_stream_alloc(gx_device_pdf *pdev, client_name_t cname)
 	gs_alloc_struct(mem, cos_stream_t, &st_cos_object, cname);
 
     cos_object_init((cos_object_t *)pcs, pdev, &cos_stream_procs);
+    pcs->md5_valid = 0;
+    gs_md5_init(&pcs->md5);
+    memset(&pcs->hash, 0x00, 16);
     return pcs;
 }
 
@@ -1232,12 +1301,38 @@ cos_stream_release(cos_object_t *pco, client_name_t cname)
     cos_dict_release(pco, cname);
 }
 
+static int hash_cos_stream(gs_memory_t *mem, const cos_object_t *pco, FILE *sfile, uint64_t *hash)
+{
+    const cos_stream_t *pcs = (const cos_stream_t *)pco;
+    cos_stream_piece_t *pcsp = pcs->pieces;
+    byte *ptr;
+    long position_save = ftell(sfile);
+    int result;
+    gs_md5_state_t md5;
+
+    gs_md5_init(&md5);
+    
+    while(pcsp) {
+	ptr = gs_malloc(mem, sizeof (byte), pcsp->size, "hash_cos_stream");
+	fseek(sfile, pcsp->position, SEEK_SET);
+	if (fread(ptr, 1, pcsp->size, sfile) != pcsp->size) {
+	    result = gs_note_error(gs_error_ioerror);
+	    return result;
+	}
+	gs_md5_append(&md5, ptr, pcsp->size);
+	gs_free(mem, ptr, sizeof (byte), pcsp->size, "hash_cos_stream");
+	pcsp = pcsp->next;
+    }
+    fseek(sfile, position_save, SEEK_SET);
+    gs_md5_finish(&md5, (gs_md5_byte_t *)hash);
+    return 0;
+}
+
 static int
 cos_stream_equal(const cos_object_t *pco0, const cos_object_t *pco1, gx_device_pdf *pdev)
 {
     const cos_stream_t *pcs0 = (const cos_stream_t *)pco0;
     const cos_stream_t *pcs1 = (const cos_stream_t *)pco1;
-    bool result = false;
     int code;
 
     code = cos_dict_equal(pco0, pco1, pdev);
@@ -1245,45 +1340,16 @@ cos_stream_equal(const cos_object_t *pco0, const cos_object_t *pco1, gx_device_p
 	return code;
     if (!code)
 	return false;
-    {
-	/* fixme : this assumes same segmentation for both streams.
-	   In general it is not true. */
-	FILE *sfile = pdev->streams.file;
-	cos_stream_piece_t *pcsp0 = pcs0->pieces, *pcsp1 = pcs1->pieces;
-	long position_save = ftell(sfile);
-
-	for (; pcsp0 && pcsp1; pcsp0 = pcsp0->next, pcsp1 = pcsp1->next) {
-	    long position0 = pcsp0->position;
-	    long position1 = pcsp1->position;
-	    uint size0 = pcsp0->size;
-	    uint size1 = pcsp1->size;
-	    byte buf0[512], buf1[sizeof(buf0)];
-
-	    if (size0 != size1)
-		goto notequal;
-	    for(; size0; position0 += size1, position1 += size1, size0 -= size1) {
-		size1 = min(sizeof(buf0), size0);
-		fseek(sfile, position0, SEEK_SET);
-		if (fread(buf0, 1, size1, sfile) != size1) {
-		    result = gs_note_error(gs_error_ioerror);
-		    goto notequal;
-		}
-		fseek(sfile, position1, SEEK_SET);
-		if (fread(buf1, 1, size1, sfile) != size1) {
-		    result = gs_note_error(gs_error_ioerror);
-		    goto notequal;
-		}
-		if (memcmp(buf0, buf1, size1))
-		    goto notequal;
-	    }
-	}
-	if (pcsp0 || pcsp1)
-	    goto notequal;
-	result = true;
-notequal:
-	fseek(sfile, position_save, SEEK_SET);
-	return result;
+    if (!pco0->md5_valid) {
+	hash_cos_stream(pdev->memory, pco0, pdev->streams.file, (uint64_t *)&pcs0->hash);
     }
+    if (!pco1->md5_valid) {
+	hash_cos_stream(pdev->memory, pco1, pdev->streams.file, (uint64_t *)&pcs1->hash);
+    }
+    if (memcmp(&pcs0->hash, &pcs1->hash, 16) == 0)
+	return true;
+    return false;
+
 }
 
 /* Find the total length of a stream. */
@@ -1488,6 +1554,7 @@ cos_write_stream_process(stream_state * st, stream_cursor_read * pr,
     int code;
 
     stream_write(target, pr->ptr + 1, count);
+    gs_md5_append(&ss->pcs->md5, pr->ptr + 1, count);
     pr->ptr = pr->limit;
     sflush(target);
     code = cos_stream_add(ss->pcs, (uint)(stell(pdev->streams.strm) - start_pos));
@@ -1501,6 +1568,8 @@ cos_write_stream_close(stream *s)
 
     sflush(s);
     status = s_close_filters(&ss->target, ss->pdev->streams.strm);
+    gs_md5_finish(&ss->pcs->md5, (gs_md5_byte_t *)ss->pcs->hash);
+    ss->pcs->md5_valid = 1;
     return (status < 0 ? status : s_std_close(s));
 }
 
@@ -1526,6 +1595,9 @@ cos_write_stream_alloc(cos_stream_t *pcs, gx_device_pdf *pdev,
 	goto fail;
     ss->template = &cos_write_stream_template;
     ss->pcs = pcs;
+    ss->pcs->md5_valid = 0;
+    gs_md5_init(&ss->pcs->md5);
+    memset(&ss->pcs->hash, 0x00, 16);
     ss->pdev = pdev;
     ss->s = s;
     ss->target = pdev->streams.strm; /* not s->strm */
