@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpdtt.c 10723 2010-02-08 16:45:47Z ken $ */
+/* $Id: gdevpdtt.c 11988 2011-01-03 14:33:56Z ken $ */
 /* Text processing for pdfwrite. */
 #include "math_.h"
 #include "string_.h"
@@ -96,6 +96,7 @@ pdf_text_set_cache(gs_text_enum_t *pte, const double *pw,
 {
     pdf_text_enum_t *const penum = (pdf_text_enum_t *)pte;
     gx_device_pdf *pdev = (gx_device_pdf *)pte->dev;
+    gs_matrix m;
 
     if (pdev->type3charpath)
 	return gs_text_set_cache(penum->pte_default, pw, control);
@@ -103,7 +104,12 @@ pdf_text_set_cache(gs_text_enum_t *pte, const double *pw,
     switch (control) {
     case TEXT_SET_CHAR_WIDTH:
     case TEXT_SET_CACHE_DEVICE:
-	gs_distance_transform(pw[0], pw[1], &ctm_only(pte->pis), &pdev->char_width);
+	/* See comments in pdf_text_process. We are using a 100x100 matrix
+	 * NOT the identity, but we want the cache device values to be in
+	 * font co-ordinate space, so we need to undo that scale here.
+	 */
+	gs_matrix_scale(&ctm_only(pte->pis), .01, .01, &m);
+	gs_distance_transform(pw[0], pw[1], &m, &pdev->char_width);
 	break;
     case TEXT_SET_CACHE_DEVICE2:
 	/*
@@ -112,7 +118,12 @@ pdf_text_set_cache(gs_text_enum_t *pte, const double *pw,
 	 * width for Widths array. Therefore we don't check 
 	 * gs_rootfont(pgs)->WMode and don't use pw[6:7].
 	 */
-	gs_distance_transform(pw[0], pw[1], &ctm_only(pte->pis), &pdev->char_width);
+	/* See comments in pdf_text_process. We are using a 100x100 matrix
+	 * NOT the identity, but we want the cache device values to be in
+	 * font co-ordinate space, so we need to undo that scale here.
+	 */
+	gs_matrix_scale(&ctm_only(pte->pis), .01, .01, &m);
+	gs_distance_transform(pw[0], pw[1], &m, &pdev->char_width);
 	if (penum->cdevproc_callout) {
 	    memcpy(penum->cdevproc_result, pw, sizeof(penum->cdevproc_result));
 	    return 0;
@@ -178,6 +189,23 @@ pdf_text_set_cache(gs_text_enum_t *pte, const double *pw,
 	    code = gx_clip_to_rectangle(penum_s->pgs, &clip_box);
 	    if (code < 0)
 		return code;
+
+	    /* See comments in pdf_text_process. We are using a 100x100 matrix
+	     * NOT the identity, but we want the cache device values to be in
+	     * font co-ordinate space, so we need to undo that scale here. We
+	     * can't do it above, where we take any scaling from the BuildChar
+	     * into account, because that would get the clip path wrong, that
+	     * needs to be in the 100x100 space so that it doesn't clip
+	     * out marking operations.
+	     */
+	    gs_matrix_scale(&ctm_only(penum_s->pgs), .01, .01, &m);
+	    for (i = 0; i < narg; i += 2) {
+		gs_point p;
+
+		gs_point_transform(pw[i], pw[i + 1], &m, &p);
+		pw1[i] = p.x;
+		pw1[i + 1] = p.y;
+	    }
 	    code = pdf_set_charproc_attrs(pdev, pte->current_font, 
 			pw1, narg, control, penum->output_char_code);
 	    if (code < 0)
@@ -209,6 +237,17 @@ pdf_text_set_cache(gs_text_enum_t *pte, const double *pw,
 	       to go with the default implementation.
 	       Need to restore the correct CTM and add
 	       changes, which the charproc possibly did. */
+	    /* See comments in pdf_text_process. We are using a 100x100 matrix
+	     * NOT the identity,  so we need to undo that scale here.
+	     */
+	    gs_matrix_scale(&ctm_only(penum->pis), .01, .01, (gs_matrix *)&ctm_only(penum->pis));
+	    /* We also scaled the page height and width. Because we
+	     * don't go through the accumulator 'close' in pdf_text_process 
+	     * we must also undo that scale.
+	     */
+	    pdev->width /= 100;
+	    pdev->height /= 100;
+
 	    gs_matrix_multiply((gs_matrix *)&pdev->charproc_ctm, (gs_matrix *)&penum->pis->ctm, &m);
 	    gs_matrix_fixed_from_matrix(&penum->pis->ctm, &m);
 	    penum->charproc_accum = false;
@@ -583,7 +622,8 @@ alloc_font_cache_elem_arrays(gx_device_pdf *pdev, pdf_font_cache_elem_t *e,
     e->num_chars = num_chars;
     e->num_widths = num_widths;
     memset(e->glyph_usage, 0, len);
-    memset(e->real_widths, 0, num_widths * sizeof(*e->real_widths));
+    if (e->real_widths != NULL)
+	memset(e->real_widths, 0, num_widths * sizeof(*e->real_widths));
     return 0;
 }
 
@@ -759,6 +799,37 @@ pdf_font_orig_matrix(const gs_font *font, gs_matrix *pmat)
     }
 }
 
+/*
+ * Special version of pdf_font_orig_matrix(), that cares FDArray font's FontMatrix too.
+ * Called only by pdf_glyph_width().
+ * 'cid' is only consulted if 'font' is a CIDFontType 0 CID font.
+ */
+static int
+glyph_orig_matrix(const gs_font *font, gs_glyph cid, gs_matrix *pmat)
+{
+    int code = pdf_font_orig_matrix(font, pmat);
+    if (code >= 0) {
+	if (font->FontType == ft_CID_encrypted) {
+	    int fidx;
+
+	    if (cid < GS_MIN_CID_GLYPH)
+		cid = GS_MIN_CID_GLYPH;
+	    code = ((gs_font_cid0 *)font)->cidata.glyph_data((gs_font_base *)font,
+				cid, NULL, &fidx);
+	    if (code < 0) {
+		code = ((gs_font_cid0 *)font)->cidata.glyph_data((gs_font_base *)font,
+				(gs_glyph)GS_MIN_CID_GLYPH, NULL, &fidx);
+	    }
+	    if (code >= 0) {
+		gs_matrix_multiply(&(gs_cid0_indexed_font(font, fidx)->FontMatrix),
+				pmat, pmat);
+	    }
+	}
+    }
+    return code;
+}
+
+
 /* 
  * Check the Encoding compatibility 
  */
@@ -1092,14 +1163,20 @@ pdf_make_font3_resource(gx_device_pdf *pdev, gs_font *font,
     if (pdfont->u.simple.s.type3.Resources == NULL)
 	return_error(gs_error_VMerror);
     /* Adobe viewers have a precision problem with small font matrices : */
-    while (any_abs(pdfont->u.simple.s.type3.FontMatrix.xx) < 0.001 &&
-	   any_abs(pdfont->u.simple.s.type3.FontMatrix.xy) < 0.001 &&
-	   any_abs(pdfont->u.simple.s.type3.FontMatrix.yx) < 0.001 &&
-	   any_abs(pdfont->u.simple.s.type3.FontMatrix.yy) < 0.001) {
-	pdfont->u.simple.s.type3.FontMatrix.xx *= 10;
-	pdfont->u.simple.s.type3.FontMatrix.xy *= 10;
-	pdfont->u.simple.s.type3.FontMatrix.yx *= 10;
-	pdfont->u.simple.s.type3.FontMatrix.yy *= 10;
+    /* Don't perform this test if all entries are 0, leads to infinite loop! */
+    if (pdfont->u.simple.s.type3.FontMatrix.xx != 0.0 ||
+	pdfont->u.simple.s.type3.FontMatrix.xy != 0.0 ||
+	pdfont->u.simple.s.type3.FontMatrix.yx != 0.0 ||
+	pdfont->u.simple.s.type3.FontMatrix.yy != 0.0) {
+	while (any_abs(pdfont->u.simple.s.type3.FontMatrix.xx) < 0.001 &&
+	       any_abs(pdfont->u.simple.s.type3.FontMatrix.xy) < 0.001 &&
+	       any_abs(pdfont->u.simple.s.type3.FontMatrix.yx) < 0.001 &&
+	       any_abs(pdfont->u.simple.s.type3.FontMatrix.yy) < 0.001) {
+	    pdfont->u.simple.s.type3.FontMatrix.xx *= 10;
+	    pdfont->u.simple.s.type3.FontMatrix.xy *= 10;
+	    pdfont->u.simple.s.type3.FontMatrix.yx *= 10;
+	    pdfont->u.simple.s.type3.FontMatrix.yy *= 10;
+	}
     }
     *ppdfont = pdfont;
     return 0;
@@ -1222,8 +1299,14 @@ pdf_make_font_resource(gx_device_pdf *pdev, gs_font *font,
 	    }
 	}
     }
-    if (font->FontType == ft_encrypted || font->FontType == ft_encrypted2 || 
-	font->FontType == ft_TrueType) {
+    if (font->FontType == ft_encrypted || font->FontType == ft_encrypted2
+	|| (font->FontType == ft_TrueType && pdev->ForOPDFRead)) {
+	/* Removed the addition of Encodings to TrueType fonts as we always write
+	 * these with the Symbolic flag set. The comment below explains why we
+	 * previously wrote these, but as the comment notes this was incorrect.
+	 * Removed as it is causing preflight problems, and is specifically
+	 * disallowed with PDF/A output.
+	 */
         /*
 	 * We write True Types with Symbolic flag set.
 	 * PDF spec says that "symbolic font should not specify Encoding entry"
@@ -1246,7 +1329,6 @@ pdf_make_font_resource(gx_device_pdf *pdev, gs_font *font,
 	(code = font_alloc(pdev, &pdfont, base_font->id, pfd)) < 0
 	)
 	return code;
-    code = 1;
 
     if (!pdf_is_CID_font(font)) {
 	pdfont->u.simple.BaseEncoding = BaseEncoding;
@@ -1274,6 +1356,8 @@ pdf_make_font_resource(gx_device_pdf *pdev, gs_font *font,
 	    widths.Width.w = 0;
 	    code = pdf_glyph_widths(pdfont, font->WMode, notdef_glyph,
 		 font, &widths, cdevproc_result);
+	    if (code < 0)
+		return code;
 	    w[0] = widths.Width.w;
 	    pdfont->used[0] |= 0x80;
 	}
@@ -1585,17 +1669,30 @@ do_unknown:
 			return_error(gs_error_unregistered);
 		    }
 		}
+	    }
 	}
-    }
 	if (pdfont != NULL)
 	    pdfont->u.simple.last_reserved_char = last_reserved_char;
 	/* Change glyphs to char codes in the text : */
 	for (i = 0; i < pstr->size; i++) {
-	    /* Picked up by Coverity, if pdfont is NULL then the call would dereference it */
+	    /* Picked up by Coverity, if pdfont is NULL then the call might dereference it */
 	    if (pdfont != NULL) {
 		/* A trick : pdf_reserve_char_code_in_pdfont here simply encodes with cgp. */
 		ch = pdf_reserve_char_code_in_pdfont(pdfont, cgp, gdata[i], &pdfont->u.simple.last_reserved_char);
 		pstr->data[i] = ch;
+	    } else {
+		/* So if pdffont is NULL, do the 'trick' code mentioned above.
+		 * If that fails (I believe it shouod not), then return an error.
+		 */
+		int j;
+
+		for (j = 0; j < cgp->num_all_chars; j++)
+		    if (cgp->s[j].glyph == gdata[i])
+			break;
+		if (j < cgp->num_all_chars)
+		    pstr->data[i] = cgp->s[j].chr;
+		else
+		    return_error(gs_error_unregistered);
 	    }
 	}
 	return 0;
@@ -1688,13 +1785,13 @@ pdf_obtain_font_resource_encoded(gx_device_pdf *pdev, gs_font *font,
 	code = pdf_attached_font_resource(pdev, base_font, ppdfont, NULL, NULL, NULL, NULL);
 	if (code < 0)
 	    return code;
-	if (*ppdfont != NULL && base_font != font) {
+	if (base_font != font) {
 	    if (pdfont_not_allowed == *ppdfont)
 		*ppdfont = NULL;	
-	    else if(!pdf_is_compatible_encoding(pdev, *ppdfont, 
+	}
+	if(*ppdfont != NULL && !pdf_is_compatible_encoding(pdev, *ppdfont, 
 				    base_font, cgp->s, cgp->num_all_chars))
 		*ppdfont = NULL;
-	}
 	if (*ppdfont == NULL || *ppdfont == pdfont_not_allowed) {
 	    pdf_resource_type_t type = 
 		(pdf_is_CID_font(base_font) ? resourceCIDFont 
@@ -2098,10 +2195,14 @@ pdf_update_text_state(pdf_text_process_state_t *ppts,
 
 	/* Sort out any pending glyphs */
 	code = pdf_set_PaintType0_params(pdev, pis, size, scaled_width, &ppts->values);
+	if (code < 0)
+	    return code;
 
 	pis->line_params.half_width = scaled_width / 2;
 	code = pdf_set_text_process_state(pdev, (const gs_text_enum_t *)penum,
 				      ppts);
+	if (code < 0)
+	    return code;
 
 	pis->line_params.half_width = saved_width;
     } else {
@@ -2133,6 +2234,9 @@ pdf_set_text_process_state(gx_device_pdf *pdev,
 	int code;
 
 	code = pdf_open_contents(pdev, PDF_IN_STRING);
+	if (code < 0)
+	    return code;
+
 	code = pdf_prepare_stroke(pdev, pis);
 	if (code >= 0) 
 	    code = gdev_vector_prepare_stroke((gx_device_vector *)pdev,
@@ -2224,10 +2328,10 @@ pdf_glyph_widths(pdf_font_resource_t *pdfont, int wmode, gs_glyph glyph,
 
     if (ofont->FontType == ft_composite)
 	return_error(gs_error_unregistered); /* Must not happen. */
-    code = pdf_font_orig_matrix((const gs_font *)cfont, &scale_c);
+    code = glyph_orig_matrix((const gs_font *)cfont, glyph, &scale_c);
     if (code < 0)
 	return code;
-    code = pdf_font_orig_matrix(ofont, &scale_o);
+    code = glyph_orig_matrix(ofont, glyph, &scale_o);
     if (code < 0)
 	return code;
     gs_matrix_scale(&scale_c, 1000.0, 1000.0, &scale_c);
@@ -2397,7 +2501,7 @@ pdf_choose_output_glyph_hame(gx_device_pdf *pdev, pdf_text_enum_t *penum, gs_con
 	p = (byte *)gs_alloc_string(pdev->pdf_memory, gnstr->size, "pdf_text_set_cache");
 	if (p == NULL)
 	    return_error(gs_error_VMerror);
-	sprintf(buf, "g%04x", glyph & 0xFFFF);
+	sprintf(buf, "g%04x", (unsigned int)(glyph & 0xFFFF));
 	memcpy(p, buf, 5);
 	gnstr->data = p;
     } else {
@@ -2474,6 +2578,7 @@ pdf_text_process(gs_text_enum_t *pte)
     if (penum->current_font->FontType == ft_user_defined && (penum->text.operation & TEXT_DO_ANY_CHARPATH)
 	&& !pdev->type3charpath) {
 	pdev->type3charpath = true;
+	if (!penum->charproc_accum)
 	goto default_impl;
     }
 
@@ -2507,11 +2612,17 @@ pdf_text_process(gs_text_enum_t *pte)
 		memcpy(KeyName, penum->current_font->key_name.chars, len);
 		KeyName[len] = 0x00;
 
-		eprintf4("ERROR: Page %d used undefined glyph '%s' from type 3 font '%s', key '%s'\n",
-		    pdev->next_page, glyph, FontName, KeyName);
+		emprintf4(pdev->memory,
+                          "ERROR: Page %d used undefined glyph '%s' from type 3 font '%s', key '%s'\n",
+		          pdev->next_page, glyph, FontName, KeyName);
 	        stream_puts(pdev->strm, "0 0 0 0 0 0 d1\n");
 	    }
 	    
+	    /* See below, we scaled the device height and width to prevent 
+	     * clipping of the CharProc operations, now we need to undo that.
+	     */
+	    pdev->width /= 100;
+	    pdev->height /= 100;
 	    code = pdf_end_charproc_accum(pdev, penum->current_font, penum->cgp, 
 			pte_default->returned.current_glyph, penum->output_char_code, &gnstr);
 	    if (code < 0)
@@ -2522,7 +2633,10 @@ pdf_text_process(gs_text_enum_t *pte)
 		return code;
 	    gs_text_release(pte_default, "pdf_text_process");
 	    penum->pte_default = 0;
-	    goto top;
+	    if (!pdev->type3charpath)
+		goto top;
+	    else
+		goto default_impl;
 	}
 	pdev->pte = pte_default; /* CAUTION: See comment in gdevpdfx.h . */
 	code = gs_text_process(pte_default);
@@ -2543,6 +2657,20 @@ pdf_text_process(gs_text_enum_t *pte)
 		code = pdf_start_charproc_accum(pdev);
 		if (code < 0)
 		    return code;
+
+		/* We need to give FreeType some room for accuracy when 
+		 * retrieving the outline. We will use a scale factor of 100
+		 * (see below). Because this appears to the regular path
+		 * handling code as if it were at the page level, the co-ords
+		 * would be clipped frequently, so we temporarily hack the
+		 * width and height of the device here to ensure it doesn't.
+		 * Note that this requires some careful manipulation of the 
+		 * CTM in various places, which want numbers in the font
+		 * co-ordinate space, not the scaled user space.
+		 */
+		pdev->width *= 100;
+		pdev->height *= 100;
+
 		pdf_viewer_state_from_imager_state(pdev, pte->pis, pte->pdcolor);
 		/* Set line params to unallowed values so that
 		   they'll synchronize with writing them out on the first use. 
@@ -2562,7 +2690,16 @@ pdf_text_process(gs_text_enum_t *pte)
 		   executed gsave, so we are safe to change CTM now.
 		   Note that BuildChar may change CTM before calling setcachedevice. */
 		gs_make_identity(&m);
+		/* See comment above, we actually want to use a scale factor
+		 * of 100 in order to give FreeType some room in the fixed
+		 * precision calculations when retrieing the outline. So in
+		 * fact we don't use the identity CTM, but a 100x100 matrix
+		 * Originally tried 1000, but that was too likely to cause
+		 * clipping or arithmetic overflow.
+		 */
+		gs_matrix_scale(&m, 100, 100, &m);
 		gs_matrix_fixed_from_matrix(&penum->pis->ctm, &m);
+
 		/* Choose a character code to use with the charproc. */
 		code = pdf_choose_output_char_code(pdev, penum, &penum->output_char_code);
 		if (code < 0)
