@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevpsdi.c 9738 2009-05-12 10:10:53Z ken $ */
+/* $Id: gdevpsdi.c 11778 2010-10-08 13:42:41Z ken $ */
 /* Image compression for PostScript and PDF writers */
 #include "stdio_.h"		/* for jpeglib.h */
 #include "jpeglib_.h"		/* for sdct.h */
@@ -29,6 +29,7 @@
 #include "slzwx.h"
 #include "spngpx.h"
 #include "szlibx.h"
+#include "gsicc_manage.h"
 #ifdef USE_LDF_JB2
 #include "sjbig2_luratech.h"
 #endif
@@ -131,7 +132,8 @@ choose_DCT_params(gx_device *pdev, const gs_color_space *pcs,
     gx_device_retain((gx_device *)&mdev, true);	/* prevent freeing */
     set_linear_color_bits_mask_shift((gx_device *)&mdev);
     mdev.color_info.separable_and_linear = GX_CINFO_SEP_LIN;
-
+    /* Set mem device icc profile */
+    gsicc_init_device_profile((const gs_state *) pis, (gx_device *) &mdev);
     if (pis) {
 	/* Check for an RGB-like color space.  
 	   To recognize that we make a matrix as it were a linear operator,
@@ -215,7 +217,6 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
     gx_device_psdf *pdev = pbw->dev;
     gs_memory_t *mem = pdev->v_memory;
     const stream_template *template = pdip->filter_template;
-    const stream_template *orig_template = template;
     const stream_template *lossless_template =
 	(pdev->params.UseFlateCompression &&
 	 pdev->version >= psdf_version_ll3 ?
@@ -246,16 +247,16 @@ setup_image_compression(psdf_binary_writer *pbw, const psdf_image_params *pdip,
 	 * called 2 times with different values of the 'lossless' argument.
 	 */
         if (lossless) {
-            orig_template = template = lossless_template;
+            template = lossless_template;
         } else if (template == NULL || template == &s_zlibE_template || 
 		   template == &s_LZWE_template) {
-            orig_template = template = &s_DCTE_template;
+            template = &s_DCTE_template;
         }
 	dict = pdip->ACSDict;
     } else if (!lossless)
 	return gs_error_rangecheck; /* Reject the alternative stream. */   
     if (pdev->version < psdf_version_ll3 && template == &s_zlibE_template)
-	orig_template = template = lossless_template;
+	template = lossless_template;
     if (dict != NULL) /* Some interpreters don't supply filter parameters. */
 	gs_c_param_list_read(dict);	/* ensure param list is in read mode */
     if (template == 0)	/* no compression */
@@ -452,9 +453,11 @@ psdf_is_converting_image_to_RGB(const gx_device_psdf * pdev,
 		const gs_imager_state * pis, const gs_pixel_image_t * pim)
 {
     return pdev->params.ConvertCMYKImagesToRGB &&
-	    pis != 0 &&
-	    gs_color_space_get_index(pim->ColorSpace) ==
-	    gs_color_space_index_DeviceCMYK;
+	    pis != 0 && pim->ColorSpace && 
+	    (gs_color_space_get_index(pim->ColorSpace) == gs_color_space_index_DeviceCMYK ||
+	    (gs_color_space_get_index(pim->ColorSpace) == gs_color_space_index_ICC
+	    && gsicc_get_default_type(pim->ColorSpace->cmm_icc_profile_data) ==
+	    gs_color_space_index_DeviceCMYK));
 }
 
 static inline void 
@@ -594,7 +597,7 @@ psdf_setup_image_filters(gx_device_psdf * pdev, psdf_binary_writer * pbw,
 	    gs_memory_t *mem = pdev->v_memory;
 
 	    /* {csrc} decref old colorspace? */
-	    rc_decrement_only(pim->ColorSpace, "psdf_setup_image_filters");
+	    rc_decrement_only_cs(pim->ColorSpace, "psdf_setup_image_filters");
 	    pim->ColorSpace = gs_cspace_new_DeviceRGB(mem);
 	}
 	if (params.Depth == -1)
