@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gdevx.c 10623 2010-01-19 21:38:16Z hintak $ */
+/* $Id: gdevx.c 11187 2010-05-05 18:14:21Z robin $ */
 /* X Windows driver for Ghostscript library */
 #include "gx.h"			/* for gx_bitmap; includes std.h */
 #include "math_.h"
@@ -580,7 +580,7 @@ x_copy_image(gx_device_X * xdev, const byte * base, int sourcex, int raster,
 	    xdev->image.bytes_per_line = 0;
 	xdev->image.bits_per_pixel = depth;
 	if (XInitImage(&xdev->image) == 0){
-	    errprintf("XInitImage failed in x_copy_image.\n");
+	    errprintf(xdev->memory, "XInitImage failed in x_copy_image.\n");
 	    return_error(gs_error_unknownerror);
 	}
 	XPutImage(xdev->dpy, xdev->dest, xdev->gc, &xdev->image,
