@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gstrans.h 9666 2009-04-20 19:16:24Z mvrhel $ */
+/* $Id: gstrans.h 11947 2010-12-10 19:50:53Z mvrhel $ */
 /* Transparency definitions and interface */
 
 #ifndef gstrans_INCLUDED
@@ -32,7 +32,11 @@ typedef enum {
     PDF14_END_TRANS_GROUP,
     PDF14_BEGIN_TRANS_MASK,
     PDF14_END_TRANS_MASK,
-    PDF14_SET_BLEND_PARAMS
+    PDF14_SET_BLEND_PARAMS,
+    PDF14_PUSH_TRANS_STATE,
+    PDF14_POP_TRANS_STATE,
+    PDF14_PUSH_SMASK_COLOR,
+    PDF14_POP_SMASK_COLOR
 } pdf14_compositor_operations;
 
 #define PDF14_OPCODE_NAMES \
@@ -43,7 +47,11 @@ typedef enum {
     "PDF14_END_TRANS_GROUP  ",\
     "PDF14_BEGIN_TRANS_MASK ",\
     "PDF14_END_TRANS_MASK   ",\
-    "PDF14_SET_BLEND_PARAMS "\
+    "PDF14_SET_BLEND_PARAMS ",\
+    "PDF14_PUSH_TRANS_STATE ",\
+    "PDF14_POP_TRANS_STATE  ",\
+    "PDF14_PUSH_SMASK_COLOR ",\
+    "PDF14_POP_SMASK_COLOR  "\
 }
 
 /* Bit definitions for serializing PDF 1.4 parameters */
@@ -81,7 +89,12 @@ struct gs_pdf14trans_params_s {
     int Background_components;
     bool function_is_identity;
     float Background[GS_CLIENT_COLOR_MAX_COMPONENTS];
-    float GrayBackground;
+    float GrayBackground;  /* This is used to determine if the 
+                              softmask's bbox needs to be adjusted
+                              to the parent groups bbox.  Since
+                              the soft mask can affect areas 
+                              outside its own groups bounding
+                              box in such a case */
     gs_function_t *transfer_function;
     byte transfer_fn[MASK_TRANSFER_FUNCTION_SIZE];
     /* Individual transparency parameters */
@@ -96,9 +109,13 @@ struct gs_pdf14trans_params_s {
 	bool overprint_mode;
     bool idle; /* For clist reader.*/
     uint mask_id; /* For clist reader.*/
-    bool SMask_is_CIE;
     int group_color_numcomps;
     gs_transparency_color_t group_color;
+    int64_t icc_hash;
+    cmm_profile_t *iccprofile;               /* The profile  */
+    bool crop_blend_params;  /* This is used when the blend params are updated
+                                during a transparency group push */
+
 };
 
 #ifndef gs_pdf14trans_params_DEFINED
@@ -179,6 +196,22 @@ int gx_end_transparency_mask(gs_imager_state * pis, gx_device * pdev,
 
 int gx_discard_transparency_layer(gs_imager_state *pis);
 
+/* These are used for watching for q Smask Q events.  We need to 
+   send special compositor commands to keep the bands in sync
+   with the current softmask during clist rendering.  Like the
+   other transparency operations the gs functions occur on the
+   clist writer side and the gx functions occur on the
+   clist reader side */
+
+int gs_push_transparency_state(gs_state *pgs);
+
+int gs_pop_transparency_state(gs_state *pgs);
+
+int gx_push_transparency_state(gs_imager_state * pis, gx_device * pdev);
+
+int gx_pop_transparency_state(gs_imager_state * pis, gx_device * pdev);
+
+
 /*
  * Verify that a compositor data structure is for the PDF 1.4 compositor.
  */
