@@ -10,7 +10,7 @@
    or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gxblend.c 9666 2009-04-20 19:16:24Z mvrhel $ */
+/* $Id: gxblend.c 11947 2010-12-10 19:50:53Z mvrhel $ */
 /* PDF 1.4 blending functions */
 
 #include "memory_.h"
@@ -18,6 +18,9 @@
 #include "gstparam.h"
 #include "gxblend.h"
 #include "gxcolor2.h"
+#include "gsicc_cache.h"
+#include "gsicc_manage.h"
+
 
 typedef int art_s32;
 
@@ -34,184 +37,191 @@ extern unsigned int clist_band_count;
 /* Note, data is planar */
 
 void 
-Smask_Luminosity_Mapping(int num_rows, int num_cols, int n_chan, int row_stride, 
-                         int plane_stride, byte *dst, const byte *src, bool isadditive,
-                         bool SMask_is_CIE, gs_transparency_mask_subtype_t SMask_SubType)
+smask_luminosity_mapping(int num_rows, int num_cols, int n_chan, int row_stride, 
+                         int plane_stride, byte *src, const byte *dst, bool isadditive,
+                         gs_transparency_mask_subtype_t SMask_SubType)
 {
-
     int x,y;
     int mask_alpha_offset,mask_C_offset,mask_M_offset,mask_Y_offset,mask_K_offset;
     int mask_R_offset,mask_G_offset,mask_B_offset;
     byte *dstptr;
 
 #if RAW_DUMP
-
     dump_raw_buffer(num_rows, row_stride, n_chan,
                 plane_stride, row_stride, 
                 "Raw_Mask", src);
 
     global_index++;
-
 #endif
-
     dstptr = dst;
-
-    /* If we are CIE AND subtype is Luminosity then we should just grab the Y channel */
-
-    if ( SMask_is_CIE && SMask_SubType == TRANSPARENCY_MASK_Luminosity ){
-
- 
+    /* If subtype is Luminosity then we should just grab the Y channel */
+    if ( SMask_SubType == TRANSPARENCY_MASK_Luminosity ){
         memcpy(dst, &(src[plane_stride]), plane_stride);
         return;
-
     }
-
     /* If we are alpha type, then just grab that */ 
     /* We need to optimize this so that we are only drawing alpha in the rect fills */
-
     if ( SMask_SubType == TRANSPARENCY_MASK_Alpha ){
-
         mask_alpha_offset = (n_chan - 1) * plane_stride;
         memcpy(dst, &(src[mask_alpha_offset]), plane_stride);
         return;
-
     }
-
     /* To avoid the if statement inside this loop, 
     decide on additive or subractive now */
-
-    if (isadditive || n_chan == 2)
-    {
-
+    if (isadditive || n_chan == 2) {
         /* Now we need to split Gray from RGB */
-
-        if( n_chan == 2 )
-        {
+        if( n_chan == 2 ) {
             /* Gray Scale case */
-
            mask_alpha_offset = (n_chan - 1) * plane_stride;
            mask_R_offset = 0;
-
-            for ( y = 0; y < num_rows; y++ )
-            {
-     
+            for ( y = 0; y < num_rows; y++ ) {
                 for ( x = 0; x < num_cols; x++ ){
-
                     /* With the current design this will indicate if 
                     we ever did a fill at this pixel. if not then move on.
                     This could have some serious optimization */
-                               
 	            if (src[x + mask_alpha_offset] != 0x00) {
-
                         dstptr[x] = src[x + mask_R_offset];
-
                     } 
-
                 }
-
                dstptr += row_stride;
                mask_alpha_offset += row_stride;
                mask_R_offset += row_stride;
-
             }
-
         } else {
-
-
-
             /* RGB case */
-
            mask_R_offset = 0;
            mask_G_offset = plane_stride;
            mask_B_offset = 2 * plane_stride;
            mask_alpha_offset = (n_chan - 1) * plane_stride;
-
-            for ( y = 0; y < num_rows; y++ )
-            {
-     
+            for ( y = 0; y < num_rows; y++ ) {
                for ( x = 0; x < num_cols; x++ ){
-
                     /* With the current design this will indicate if 
                     we ever did a fill at this pixel. if not then move on */
-                               
 	            if (src[x + mask_alpha_offset] != 0x00) {
-
 	                /* Get luminosity of Device RGB value */
-
                         float temp;
-
                         temp = ( 0.30 * src[x + mask_R_offset] + 
                             0.59 * src[x + mask_G_offset] + 
                             0.11 * src[x + mask_B_offset] );
-     
                         temp = temp * (1.0 / 255.0 );  /* May need to be optimized */
 	                dstptr[x] = float_color_to_byte_color(temp);
-
                     } 
-
                 }
-
                dstptr += row_stride;
                mask_alpha_offset += row_stride;
                mask_R_offset += row_stride;
                mask_G_offset += row_stride;
                mask_B_offset += row_stride;
-
             }
-            
         }
-
     } else {
-
        /* CMYK case */
-
        mask_alpha_offset = (n_chan - 1) * plane_stride;
        mask_C_offset = 0;
        mask_M_offset = plane_stride;
        mask_Y_offset = 2 * plane_stride;
        mask_K_offset = 3 * plane_stride;
-
        for ( y = 0; y < num_rows; y++ ){
-
             for ( x = 0; x < num_cols; x++ ){
-
                 /* With the current design this will indicate if 
                 we ever did a fill at this pixel. if not then move on */
-                            
 	        if (src[x + mask_alpha_offset] != 0x00){
-
                   /* PDF spec says to use Y = 0.30 (1 - C)(1 - K) + 
                   0.59 (1 - M)(1 - K) + 0.11 (1 - Y)(1 - K) */
                     /* For device CMYK */
-
                     float temp;
-
                     temp = ( 0.30 * ( 0xff - src[x + mask_C_offset]) + 
                         0.59 * ( 0xff - src[x + mask_M_offset]) + 
                         0.11 * ( 0xff - src[x + mask_Y_offset]) ) * 
                         ( 0xff - src[x + mask_K_offset]);
-
                     temp = temp * (1.0 / 65025.0 );  /* May need to be optimized */
-
 	            dstptr[x] = float_color_to_byte_color(temp);
-
                 } 
- 
             }
-
            dstptr += row_stride;
            mask_alpha_offset += row_stride;
            mask_C_offset += row_stride;
            mask_M_offset += row_stride;
            mask_Y_offset += row_stride;
            mask_K_offset += row_stride;
-
         }
-
-
     }
+}
 
+/* soft mask gray buffer should be blended with its transparency planar data
+   during the pop for a luminosity case if we have a soft mask within a soft
+   mask.  This situation is detected in the code so that we only do this 
+   blending in those rare situations */
+void
+smask_blend(byte *src, int width, int height, int rowstride, 
+                      int planestride)
+{
+    int x, y;
+    int position;
+    byte comp, a;
+    int tmp;
+    byte bg = 0;
 
+    for (y = 0; y < height; y++) {
+        position = y * rowstride;
+        for (x = 0; x < width; x++) {
+            a = src[position + planestride];
+            if ((a + 1) & 0xfe) {
+                a ^= 0xff;
+                comp  = src[position];
+                tmp = ((bg - comp) * a) + 0x80;
+                comp += (tmp + (tmp >> 8)) >> 8;
+                src[position] = comp;
+            } else if (a == 0) {
+                src[position] = 0;
+            }
+            position+=1;
+        }
+    }
+}
+
+void smask_copy(int num_rows, int num_cols, int row_stride, 
+                        byte *src, const byte *dst)
+{
+    int y;
+    byte *dstptr,*srcptr;
+
+    dstptr = dst;
+    srcptr = src;
+    for ( y = 0; y < num_rows; y++ ) {
+        memcpy(dstptr,srcptr,num_cols);
+        dstptr += row_stride;
+        srcptr += row_stride;
+    }
+}
+
+void smask_icc(int num_rows, int num_cols, int n_chan, int row_stride, 
+                 int plane_stride, byte *src, const byte *dst, 
+                 gsicc_link_t *icclink)
+{
+    gsicc_bufferdesc_t input_buff_desc;
+    gsicc_bufferdesc_t output_buff_desc;
+
+#if RAW_DUMP
+    dump_raw_buffer(num_rows, row_stride, n_chan,
+                plane_stride, row_stride, 
+                "Raw_Mask_ICC", src);
+    global_index++;
+#endif
+/* Set up the buffer descriptors. Note that pdf14 always has
+   the alpha channels at the back end (last planes).  
+   We will just handle that here and let the CMM know 
+   nothing about it */
+
+    gsicc_init_buffer(&input_buff_desc, n_chan-1, 1,
+                  false, false, true, plane_stride, row_stride,
+                  num_rows, num_cols);
+    gsicc_init_buffer(&output_buff_desc, 1, 1,
+                  false, false, true, plane_stride, 
+                  row_stride, num_rows, num_cols);
+    /* Transform the data */
+    gscms_transform_color_buffer(icclink, &input_buff_desc, 
+                        &output_buff_desc, (void*) src, (void*) dst);
 }
 
 void
@@ -1192,10 +1202,28 @@ art_pdf_composite_group_8(byte *dst, byte *dst_alpha_g,
     }
 }
 
+/* A very simple case.  Knockout isolated group going to a parent that is not
+   a knockout.  Simply copy over everwhere where we have a non-zero alpha value */
+void
+art_pdf_knockoutisolated_group_8(byte *dst, const byte *src, int n_chan)
+{
+    int i;
+    byte src_alpha;	
+
+    src_alpha = src[n_chan];
+    if (src_alpha == 0)
+        return;
+    for (i = 0; i <= n_chan >> 2; i++) {
+        ((bits32 *) dst)[i] = ((const bits32 *)src)[i];
+    }
+}
+
 void
 art_pdf_composite_knockout_simple_8(byte *dst,
 				    byte *dst_shape,
+                                    byte *dst_tag,
 				    const byte *src,
+                                    byte tag,
 				    int n_chan, byte opacity)
 {
     byte src_shape = src[n_chan];
@@ -1240,9 +1268,11 @@ art_pdf_composite_knockout_simple_8(byte *dst,
 void
 art_pdf_composite_knockout_isolated_8(byte *dst,
 				      byte *dst_shape,
+                                      byte *dst_tag,
 				      const byte *src,
 				      int n_chan,
 				      byte shape,
+                                      byte tag,
 				      byte alpha_mask, byte shape_mask)
 {
     int tmp;
@@ -1257,6 +1287,8 @@ art_pdf_composite_knockout_isolated_8(byte *dst,
 	dst[n_chan] = (tmp + (tmp >> 8)) >> 8;
 	if (dst_shape != NULL)
 	    *dst_shape = 255;
+        if (dst_tag != NULL)
+            *dst_tag = tag;
     } else {
 	/* Use src_shape to interpolate (in premultiplied alpha space)
 	   between dst and (src, opacity). */
@@ -1290,6 +1322,9 @@ art_pdf_composite_knockout_isolated_8(byte *dst,
 	    tmp = (255 - *dst_shape) * (255 - src_shape) + 0x80;
 	    *dst_shape = 255 - ((tmp + (tmp >> 8)) >> 8);
 	}
+	if (dst_tag != NULL) {
+            *dst_tag = (*dst_tag | tag) & ~GS_UNTOUCHED_TAG;
+	}
     }
 }
 
@@ -1411,16 +1446,13 @@ art_pdf_composite_knockout_8(byte *dst,
 }
 
 #if RAW_DUMP
-
 /* Debug dump of buffer data from pdf14 device.  Saved in
    planar form with global indexing and tag information in
    file name */
-
 void
 dump_raw_buffer(int num_rows, int width, int n_chan,
                 int plane_stride, int rowstride, 
                 char filename[],byte *Buffer)
-
 {
     char full_file_name[50];
     FILE *fid;
@@ -1431,34 +1463,21 @@ dump_raw_buffer(int num_rows, int width, int n_chan,
    /* clist_band_count is incremented at every pdf14putimage */
    /* Useful for catching this thing and only dumping */
    /* during a particular band if we have a large file */
-   /* if (clist_band_count != 80) return; */
-
+   // if (clist_band_count != 1) return; 
     buff_ptr = Buffer;
-
     max_bands = ( n_chan < 57 ? n_chan : 56);   /* Photoshop handles at most 56 bands */
     sprintf(full_file_name,"%d)%s_%dx%dx%d.raw",global_index,filename,width,num_rows,max_bands);
     fid = fopen(full_file_name,"wb");
 
     for (z = 0; z < max_bands; ++z) {
-
         /* grab pointer to the next plane */
-
         buff_ptr = &(Buffer[z*plane_stride]);
-    
         for ( y = 0; y < num_rows; y++ ) {
-
             /* write out each row */
             fwrite(buff_ptr,sizeof(unsigned char),width,fid);
-
             buff_ptr += rowstride;
-
         }
-
     }
-
-	fclose(fid);
-
+    fclose(fid);
 }
-
-
 #endif
