@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gxchar.c 10603 2010-01-12 09:49:58Z masaki $ */
+/* $Id: gxchar.c 11745 2010-09-27 12:05:17Z ken $ */
 /* Default implementation of text writing */
 #include "gx.h"
 #include "memory_.h"
@@ -680,6 +680,9 @@ set_cache_device(gs_show_enum * penum, gs_state * pgs, floatp llx, floatp lly,
 	if ((code = gx_clip_to_rectangle(pgs, &clip_box)) < 0)
 	    return code;
 	gx_set_device_color_1(pgs);	/* write 1's */
+        gs_swapcolors_quick(pgs);
+	gx_set_device_color_1(pgs);	/* write 1's */
+        gs_swapcolors_quick(pgs);
 	pgs->in_cachedevice = CACHE_DEVICE_CACHING;
     }
     penum->width_status = sws_cache;
@@ -1053,8 +1056,11 @@ show_proceed(gs_show_enum * penum)
 		        /* If antialiasing is in effect, don't use xfont */
 		        if (log2_scale.x + log2_scale.y > 0)
 			    goto no_cache;
+			/* Don't use xfont for outline (stroked) fonts or
+			 * as the descendant of a type 0 font.
+			 */
 			if (pfont->ExactSize == fbit_use_outlines ||
-			    pfont->PaintType == 2
+			    pfont->PaintType == 2 || rfont->FontType == 0
 			    )
 			    goto no_cache;
 			if (pfont->BitmapWidths) {
